@@ -16,11 +16,14 @@ pub mod operator_handler;
 mod py;
 pub mod reduction;
 pub mod run_program;
-mod serialize;
+pub mod serialize;
 mod sha2;
 
 #[cfg(test)]
 mod tests;
 
+#[cfg(test)]
+mod test_ops;
+
 #[cfg(target_family = "wasm")]
 pub mod wasm;
