@@ -1,5 +1,6 @@
 use bls12_381::{G1Affine, G1Projective, Scalar};
 use num_bigint::{BigUint, Sign};
+use num_integer::Integer;
 use std::convert::TryFrom;
 use std::ops::BitAndAssign;
 use std::ops::BitOrAssign;
@@ -90,6 +91,57 @@ fn limbs_for_int(v: &Number) -> usize {
     ((v.bits() + 7) / 8) as usize
 }
 
+#[cfg(test)]
+fn limb_test_helper(bytes: &[u8]) {
+    let bigint = Number::from_signed_bytes_be(&bytes);
+    println!("{} bits: {}", &bigint, &bigint.bits());
+
+    // redundant leading zeros don't count, since they aren't stored internally
+    let expected = if bytes.len() > 0 && bytes[0] == 0 {
+        bytes.len() - 1
+    } else {
+        bytes.len()
+    };
+    assert_eq!(limbs_for_int(&bigint), expected);
+}
+
+#[test]
+fn test_limbs_for_int() {
+    limb_test_helper(&[]);
+    limb_test_helper(&[0x1]);
+    limb_test_helper(&[0x80]);
+    limb_test_helper(&[0x81]);
+    limb_test_helper(&[0x7f]);
+    limb_test_helper(&[0xff]);
+    limb_test_helper(&[0, 0xff]);
+    limb_test_helper(&[0x7f, 0xff]);
+    limb_test_helper(&[0x7f, 0]);
+    limb_test_helper(&[0x7f, 0x77]);
+
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0, 0]);
+
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x40, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x20, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x10, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x08, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x04, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x02, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x01, 0, 0, 0, 0, 0, 0]);
+
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0, 0, 0]);
+    limb_test_helper(&[0x80, 0, 0, 0, 0, 0, 0, 0]);
+}
+
 fn new_atom_and_cost(a: &mut Allocator, cost: Cost, buf: &[u8]) -> Response {
     let c = buf.len() as Cost * MALLOC_COST_PER_BYTE;
     Ok(Reduction(cost + c, a.new_atom(buf)?))
@@ -378,21 +430,23 @@ pub fn op_div(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     if a1.sign() == Sign::NoSign {
         args.first()?.err("div with 0")
     } else {
-        let q = &a0 / &a1;
-        let r = &a0 - &a1 * &q;
+        let (mut q, r) = a0.div_mod_floor(&a1);
 
-        // rust rounds division towards zero, but we want division to round
-        // toward negative infinity.
-        let q = if q.sign() == Sign::Minus && r.sign() != Sign::NoSign {
-            q - 1
-        } else {
-            q
-        };
+        // this is to preserve a buggy behavior from the initial implementation
+        // of this operator.
+        if q == (-1).into() && r != 0.into() {
+            q += 1;
+        }
         let q1 = ptr_from_number(a, &q)?;
         Ok(malloc_cost(a, cost, q1))
     }
 }
 
+pub fn op_div_deprecated(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
+    let args = Node::new(a, input);
+    args.err("div operator is deprecated")
+}
+
 pub fn op_divmod(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     let args = Node::new(a, input);
     let (a0, l0, a1, l1) = two_ints(&args, "divmod")?;
@@ -400,19 +454,7 @@ pub fn op_divmod(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response
     if a1.sign() == Sign::NoSign {
         args.first()?.err("divmod with 0")
     } else {
-        let q = &a0 / &a1;
-        let r = &a0 - &a1 * &q;
-
-        let signed_quotient =
-            (a0.sign() == Sign::Minus || a1.sign() == Sign::Minus) && a0.sign() != a1.sign();
-
-        // rust rounds division towards zero, but we want division to round
-        // toward negative infinity.
-        let (q, r) = if signed_quotient && r.sign() != Sign::NoSign {
-            (q - 1, r + &a1)
-        } else {
-            (q, r)
-        };
+        let (q, r) = a0.div_mod_floor(&a1);
         let q1 = ptr_from_number(a, &q)?;
         let r1 = ptr_from_number(a, &r)?;
 
@@ -526,7 +568,7 @@ pub fn op_ash(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     let l0 = b0.len();
     let rest = args.rest()?;
     let a1 = i32_atom(&rest.first()?, "ash")?;
-    if a1.abs() > 65535 {
+    if a1 > 65535 || a1 < -65535 {
         return args.rest()?.first()?.err("shift too large");
     }
 
@@ -537,6 +579,63 @@ pub fn op_ash(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     Ok(malloc_cost(a, cost, r))
 }
 
+#[cfg(test)]
+fn test_shift(
+    op: fn(&mut Allocator, NodePtr, Cost) -> Response,
+    a: &mut Allocator,
+    a1: &[u8],
+    a2: &[u8],
+) -> Response {
+    let args = a.null();
+    let a2 = a.new_atom(a2).unwrap();
+    let args = a.new_pair(a2, args).unwrap();
+    let a1 = a.new_atom(a1).unwrap();
+    let args = a.new_pair(a1, args).unwrap();
+    op(a, args, 10000000 as Cost)
+}
+
+#[test]
+fn test_op_ash() {
+    let mut a = Allocator::new();
+
+    assert_eq!(
+        test_shift(op_ash, &mut a, &[1], &[0x80, 0, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    assert_eq!(
+        test_shift(op_ash, &mut a, &[1], &[0x80, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    let node = test_shift(op_ash, &mut a, &[1], &[0x80, 0]).unwrap().1;
+    assert_eq!(a.atom(node), &[]);
+
+    assert_eq!(
+        test_shift(op_ash, &mut a, &[1], &[0x7f, 0, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    assert_eq!(
+        test_shift(op_ash, &mut a, &[1], &[0x7f, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    let node = test_shift(op_ash, &mut a, &[1], &[0x7f, 0]).unwrap().1;
+    // the result is 1 followed by 4064 zeroes
+    let node = a.atom(node);
+    assert_eq!(node[0], 1);
+    assert_eq!(node.len(), 4065);
+}
+
 pub fn op_lsh(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     let args = Node::new(a, input);
     check_arg_count(&args, 2, "lsh")?;
@@ -546,7 +645,7 @@ pub fn op_lsh(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     let l0 = b0.len();
     let rest = args.rest()?;
     let a1 = i32_atom(&rest.first()?, "lsh")?;
-    if a1.abs() > 65535 {
+    if a1 > 65535 || a1 < -65535 {
         return args.rest()?.first()?.err("shift too large");
     }
 
@@ -560,6 +659,48 @@ pub fn op_lsh(a: &mut Allocator, input: NodePtr, _max_cost: Cost) -> Response {
     Ok(malloc_cost(a, cost, r))
 }
 
+#[test]
+fn test_op_lsh() {
+    let mut a = Allocator::new();
+
+    assert_eq!(
+        test_shift(op_lsh, &mut a, &[1], &[0x80, 0, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    assert_eq!(
+        test_shift(op_lsh, &mut a, &[1], &[0x80, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    let node = test_shift(op_lsh, &mut a, &[1], &[0x80, 0]).unwrap().1;
+    assert_eq!(a.atom(node), &[]);
+
+    assert_eq!(
+        test_shift(op_lsh, &mut a, &[1], &[0x7f, 0, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    assert_eq!(
+        test_shift(op_lsh, &mut a, &[1], &[0x7f, 0, 0])
+            .unwrap_err()
+            .1,
+        "shift too large"
+    );
+
+    let node = test_shift(op_lsh, &mut a, &[1], &[0x7f, 0]).unwrap().1;
+    // the result is 1 followed by 4064 zeroes
+    let node = a.atom(node);
+    assert_eq!(node[0], 1);
+    assert_eq!(node.len(), 4065);
+}
+
 fn binop_reduction(
     op_name: &str,
     a: &mut Allocator,
@@ -678,21 +819,23 @@ pub fn op_softfork(a: &mut Allocator, input: NodePtr, max_cost: Cost) -> Respons
 
 lazy_static! {
     static ref GROUP_ORDER: Number = {
-        let order_as_hex = b"73EDA753299D7D483339D80809A1D80553BDA402FFFE5BFEFFFFFFFF00000001";
-        let n = BigUint::parse_bytes(order_as_hex, 16).unwrap();
+        let order_as_bytes = &[
+            0x73, 0xed, 0xa7, 0x53, 0x29, 0x9d, 0x7d, 0x48, 0x33, 0x39, 0xd8, 0x08, 0x09, 0xa1,
+            0xd8, 0x05, 0x53, 0xbd, 0xa4, 0x02, 0xff, 0xfe, 0x5b, 0xfe, 0xff, 0xff, 0xff, 0xff,
+            0x00, 0x00, 0x00, 0x01,
+        ];
+        let n = BigUint::from_bytes_be(order_as_bytes);
         n.into()
     };
 }
 
 fn mod_group_order(n: Number) -> Number {
     let order = GROUP_ORDER.clone();
-    let divisor: Number = &n / &order;
-    let remainder: Number = &n - &divisor * &order;
+    let mut remainder = n.mod_floor(&order);
     if remainder.sign() == Sign::Minus {
-        order + remainder
-    } else {
-        remainder
+        remainder += order;
     }
+    remainder
 }
 
 fn number_to_scalar(n: Number) -> Scalar {
